@@ -1,11 +1,19 @@
 # Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
-# All rights reserved.
 #
-# LIMIX is provided under a 2-clause BSD license.
-# See license.txt for the complete license.
+#Licensed under the Apache License, Version 2.0 (the "License");
+#you may not use this file except in compliance with the License.
+#You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+#Unless required by applicable law or agreed to in writing, software
+#distributed under the License is distributed on an "AS IS" BASIS,
+#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#See the License for the specific language governing permissions and
+#limitations under the License.
 
 import sys
-import scipy as SP
+import scipy as sp 
 import scipy.linalg
 import scipy.stats
 import limix
@@ -36,12 +44,16 @@ class VarianceDecomposition:
             standardize:    if True, impute missing phenotype values by mean value,
                             zero-mean and unit-variance phenotype (Boolean, default False)
         """
+
+        #check whether Y is a vector, if yes reshape
+        if (len(Y.shape)==1):
+            Y = Y[:,sp.newaxis]
         
         #create column of 1 for fixed if nothing providede
         self.N       = Y.shape[0]
         self.P       = Y.shape[1]
         self.Nt      = self.N*self.P
-        self.Iok     = ~(SP.isnan(Y).any(axis=1))
+        self.Iok     = ~(sp.isnan(Y).any(axis=1))
 
         #outsourced to handle missing values:
         if standardize:
@@ -91,7 +103,7 @@ class VarianceDecomposition:
             Y=preprocess.standardize(Y)
 
         #check that missing values match the current structure
-        assert (~(SP.isnan(Y).any(axis=1))==self.Iok).all(), 'VarianceDecomposition:: pattern of missing values needs to match Y given at initialization'
+        assert (~(sp.isnan(Y).any(axis=1))==self.Iok).all(), 'VarianceDecomposition:: pattern of missing values needs to match Y given at initialization'
 
         self.Y = Y
         self.vd.setPheno(Y)
@@ -138,7 +150,7 @@ class VarianceDecomposition:
         
         if is_noise:
             assert self.noisPos==None, 'VarianceDecomposition:: Noise term already exists'
-            K  = SP.eye(self.N)
+            K  = sp.eye(self.N)
             Kcross = None
             self.noisPos = self.n_randEffs
         else:
@@ -183,15 +195,15 @@ class VarianceDecomposition:
 
         Args:
             F:     sample design matrix for the fixed effect [N,K]
-            A:     trait design matrix for the fixed effect (e.g. SP.ones((1,P)) common effect; SP.eye(P) any effect) [L,P]
+            A:     trait design matrix for the fixed effect (e.g. sp.ones((1,P)) common effect; sp.eye(P) any effect) [L,P]
             Ftest: sample design matrix for test samples [Ntest,K]
         """
         if A==None:
-            A = SP.eye(self.P)
+            A = sp.eye(self.P)
         if F==None:
-            F = SP.ones((self.N,1))
+            F = sp.ones((self.N,1))
             if self.Ntest!=None:
-                Ftest = SP.ones((self.Ntest,1)) 
+                Ftest = sp.ones((self.Ntest,1)) 
         
         assert A.shape[1]==self.P, 'VarianceDecomposition:: A has incompatible shape'
         assert F.shape[0]==self.N, 'VarianceDecimposition:: F has incompatible shape'
@@ -290,15 +302,15 @@ class VarianceDecomposition:
             if not perturb:		n_times = 1
 
         if fixed0==None:
-            fixed0 = SP.zeros_like(self.gp.getParams()['dataTerm'])
+            fixed0 = sp.zeros_like(self.gp.getParams()['dataTerm'])
 
         for i in range(n_times):
             if init_method=='random':
                 scales1 = self._getScalesRand()
-                fixed1  = pertSize*SP.randn(fixed0.shape[0],fixed0.shape[1])
+                fixed1  = pertSize*sp.randn(fixed0.shape[0],fixed0.shape[1])
             elif perturb:
                 scales1 = scales0+pertSize*self._perturbation()
-                fixed1  = fixed0+pertSize*SP.randn(fixed0.shape[0],fixed0.shape[1])
+                fixed1  = fixed0+pertSize*sp.randn(fixed0.shape[0],fixed0.shape[1])
             else:
                 scales1 = scales0
                 fixed1  = fixed0
@@ -329,20 +341,20 @@ class VarianceDecomposition:
         
         opt_list = []
 
-        fixed0 = SP.zeros_like(self.gp.getParams()['dataTerm'])    
+        fixed0 = sp.zeros_like(self.gp.getParams()['dataTerm'])    
 
         # minimize n_times
         for i in range(n_times):
             
             scales1 = self._getScalesRand()
-            fixed1  = 1e-1*SP.randn(fixed0.shape[0],fixed0.shape[1])
+            fixed1  = 1e-1*sp.randn(fixed0.shape[0],fixed0.shape[1])
             conv = self.trainGP(fast=fast,scales0=scales1,fixed0=fixed1,lambd=lambd)
 
             if conv:
                 # compare with previous minima
                 temp=1
                 for j in range(len(opt_list)):
-                    if SP.allclose(abs(self.getScales()),abs(opt_list[j]['scales'])):
+                    if sp.allclose(abs(self.getScales()),abs(opt_list[j]['scales'])):
                         temp=0
                         opt_list[j]['counter']+=1
                         break
@@ -355,7 +367,7 @@ class VarianceDecomposition:
         
         
         # sort by LML
-        LML = SP.array([opt_list[i]['LML'] for i in range(len(opt_list))])
+        LML = sp.array([opt_list[i]['LML'] for i in range(len(opt_list))])
         index   = LML.argsort()[::-1]
         out = []
         if verbose:
@@ -399,7 +411,7 @@ class VarianceDecomposition:
         if scales==None:
             for term_i in range(self.n_randEffs):
                 n_scales = self.vd.getTerm(term_i).getNumberScales()
-                self.vd.getTerm(term_i).setScales(SP.array(SP.randn(n_scales)))
+                self.vd.getTerm(term_i).setScales(sp.array(sp.randn(n_scales)))
         elif term_num==None:
             assert scales.shape[0]==self.vd.getNumberScales(), 'incompatible shape'
             index = 0
@@ -451,7 +463,7 @@ class VarianceDecomposition:
         assert self.P>1, 'Trait covars not defined for single trait analysis'
         
         if term_i==None:
-            RV=SP.zeros((self.P,self.P))
+            RV=sp.zeros((self.P,self.P))
             for term_i in range(self.n_randEffs): RV+=self.vd.getTerm(term_i).getTraitCovar().K()
         else:
             assert term_i<self.n_randEffs, 'Term index non valid'
@@ -470,7 +482,7 @@ class VarianceDecomposition:
             estimated trait correlation coefficient matrix
         """
         cov = self.getTraitCovar(term_i)
-        stds=SP.sqrt(cov.diagonal())[:,SP.newaxis]
+        stds=sp.sqrt(cov.diagonal())[:,sp.newaxis]
         RV = cov/stds/stds.T
         return RV
     
@@ -485,13 +497,13 @@ class VarianceDecomposition:
             variance components of all random effects on all phenotypes [P, n_randEffs matrix]
         """
         if self.P>1:
-            RV=SP.zeros((self.P,self.n_randEffs))
+            RV=sp.zeros((self.P,self.n_randEffs))
             for term_i in range(self.n_randEffs):
                 RV[:,term_i] = self.getTraitCovar(term_i).diagonal()
         else:
-            RV=self.getScales()[SP.newaxis,:]**2
+            RV=self.getScales()[sp.newaxis,:]**2
         if univariance:
-            RV /= RV.sum(1)[:,SP.newaxis]
+            RV /= RV.sum(1)[:,sp.newaxis]
         return RV
 
 
@@ -508,20 +520,20 @@ class VarianceDecomposition:
         assert self.fast==False, 'Not supported for fast implementation'
         
         if self.P==1:
-            out = (2*self.getScales()[term_i])**2*self.getLaplaceCovar()[term_i,term_i]
+            out = (2*self.getScales()[term_i])**2*self._getLaplaceCovar()[term_i,term_i]
         else:
             C = self.vd.getTerm(term_i).getTraitCovar()
             n_params = C.getNumberParams()
             par_index = 0
             for term in range(term_i-1):
                 par_index += self.vd.getTerm(term_i).getNumberScales()
-            Sigma1 = self.getLaplaceCovar()[par_index:(par_index+n_params),:][:,par_index:(par_index+n_params)]
-            out = SP.zeros((self.P,self.P))
+            Sigma1 = self._getLaplaceCovar()[par_index:(par_index+n_params),:][:,par_index:(par_index+n_params)]
+            out = sp.zeros((self.P,self.P))
             for param_i in range(n_params):
                 out += C.Kgrad_param(param_i)**2*Sigma1[param_i,param_i]
                 for param_j in range(param_i):
                     out += 2*abs(C.Kgrad_param(param_i)*C.Kgrad_param(param_j))*Sigma1[param_i,param_j]
-        out = SP.sqrt(out)
+        out = sp.sqrt(out)
         return out
 
     def getVarianceCompStdErrors(self,univariance=False):
@@ -533,12 +545,12 @@ class VarianceDecomposition:
         Returns:
             standard errors on variance components [P, n_randEffs matrix]
         """
-        RV=SP.zeros((self.P,self.n_randEffs))
+        RV=sp.zeros((self.P,self.n_randEffs))
         for term_i in range(self.n_randEffs):
             RV[:,term_i] = self.getTraitCovarStdErrors(term_i).diagonal()
         var = getVarianceComps()
         if univariance:
-            RV /= var.sum(1)[:,SP.newaxis]
+            RV /= var.sum(1)[:,sp.newaxis]
         return RV
 
     """
@@ -567,7 +579,7 @@ class VarianceDecomposition:
         if self.fast==False:
             KiY = KiY.reshape(self.P,self.N).T
                 
-        Ypred = SP.zeros((self.Ntest,self.P))
+        Ypred = sp.zeros((self.Ntest,self.P))
 
         # predicting from random effects
         for term_i in use_random:
@@ -576,10 +588,10 @@ class VarianceDecomposition:
                 if Kstar==None:
                     warnings.warn('warning: random effect term %d not used for predictions as it has None cross covariance'%term_i)
                     continue 
-                term  = SP.dot(Kstar.T,KiY)
+                term  = sp.dot(Kstar.T,KiY)
                 if self.P>1:
                     C    = self.getTraitCovar(term_i)
-                    term = SP.dot(term,C)
+                    term = sp.dot(term,C)
                 else:
                     term *= self.getVarianceComps()[0,term_i]
                 Ypred += term
@@ -592,11 +604,11 @@ class VarianceDecomposition:
             if Fstar==None:
                 warnings.warn('warning: fixed effect term %d not used for predictions as it has None test sample design'%term_i)
                 continue 
-            if self.P==1:	A = SP.eye(1)
+            if self.P==1:	A = sp.eye(1)
             else:			A = self.vd.getDesign(term_i)
             Fstar = self.Fstar[term_i]
             W = weights[w_i:w_i+A.shape[0],0:1].T 
-            term = SP.dot(Fstar,SP.dot(W,A))
+            term = sp.dot(Fstar,sp.dot(W,A))
             w_i += A.shape[0]
             Ypred += term
 
@@ -619,12 +631,12 @@ class VarianceDecomposition:
         verbose = limix.getVerbose(verbose)
 
         # split samples into training and test
-        SP.random.seed(seed)
-        r = SP.random.permutation(self.Y.shape[0])
+        sp.random.seed(seed)
+        r = sp.random.permutation(self.Y.shape[0])
         nfolds = 10
-        Icv = SP.floor(((SP.ones((self.Y.shape[0]))*nfolds)*r)/self.Y.shape[0])
+        Icv = sp.floor(((sp.ones((self.Y.shape[0]))*nfolds)*r)/self.Y.shape[0])
 
-        RV = SP.zeros_like(self.Y)
+        RV = sp.zeros_like(self.Y)
 
         for fold_j in range(n_folds):
 
@@ -669,7 +681,7 @@ class VarianceDecomposition:
                 RV[Itest,:] = vc.predictPhenos()
             else:
                 warnings.warn('not converged for fold %d' % fold_j)
-                RV[Itest,:] = SP.nan
+                RV[Itest,:] = sp.nan
 
         return RV
 
@@ -684,11 +696,11 @@ class VarianceDecomposition:
             fast:   Boolean indicator denoting if fast implementation is to consider
                     if fast is None (default) and possible in the specifc situation, fast implementation is considered
         """
-        if fast==None:		fast = (self.n_randEffs==2) and (self.P>1) and (~SP.isnan(self.Y).any())
+        if fast==None:		fast = (self.n_randEffs==2) and (self.P>1) and (~sp.isnan(self.Y).any())
         elif fast:
             assert self.n_randEffs==2, 'VarianceDecomposition: for fast inference number of random effect terms must be == 2'
             assert self.P>1, 'VarianceDecomposition: for fast inference number of traits must be > 1'
-            assert not SP.isnan(self.Y).any(), 'VarianceDecomposition: fast inference available only for complete phenotype designs'
+            assert not sp.isnan(self.Y).any(), 'VarianceDecomposition: fast inference available only for complete phenotype designs'
         if fast:	self.vd.initGPkronSum()
         else:		self.vd.initGPbase()
         self.gp=self.vd.getGP()
@@ -712,45 +724,45 @@ class VarianceDecomposition:
         cov = limix.CSumCF()
         if trait_covar_type=='freeform':
             cov.addCovariance(limix.CFreeFormCF(self.P))
-            L = SP.eye(self.P)
-            diag = SP.concatenate([L[i,:(i+1)] for i in range(self.P)])
+            L = sp.eye(self.P)
+            diag = sp.concatenate([L[i,:(i+1)] for i in range(self.P)])
         elif trait_covar_type=='fixed':
             assert fixed_trait_covar!=None, 'VarianceDecomposition:: set fixed_trait_covar'
             assert fixed_trait_covar.shape[0]==self.N, 'VarianceDecomposition:: Incompatible shape for fixed_trait_covar'
             assert fixed_trait_covar.shape[1]==self.N, 'VarianceDecomposition:: Incompatible shape for fixed_trait_covar'
             cov.addCovariance(limix.CFixedCF(fixed_trait_covar))
-            diag = SP.zeros(1)
+            diag = sp.zeros(1)
         elif trait_covar_type=='diag':
             cov.addCovariance(limix.CDiagonalCF(self.P))
-            diag = SP.ones(self.P)
+            diag = sp.ones(self.P)
         elif trait_covar_type=='lowrank':
             cov.addCovariance(limix.CLowRankCF(self.P,rank))
-            diag = SP.zeros(self.P*rank)
+            diag = sp.zeros(self.P*rank)
         elif trait_covar_type=='lowrank_id':
             cov.addCovariance(limix.CLowRankCF(self.P,rank))
-            cov.addCovariance(limix.CFixedCF(SP.eye(self.P)))
-            diag = SP.concatenate([SP.zeros(self.P*rank),SP.ones(1)])
+            cov.addCovariance(limix.CFixedCF(sp.eye(self.P)))
+            diag = sp.concatenate([sp.zeros(self.P*rank),sp.ones(1)])
         elif trait_covar_type=='lowrank_diag':
             cov.addCovariance(limix.CLowRankCF(self.P,rank))
             cov.addCovariance(limix.CDiagonalCF(self.P))
-            diag = SP.concatenate([SP.zeros(self.P*rank),SP.ones(self.P)])
+            diag = sp.concatenate([sp.zeros(self.P*rank),sp.ones(self.P)])
         elif trait_covar_type=='block':
-            cov.addCovariance(limix.CFixedCF(SP.ones((self.P,self.P))))
-            diag = SP.zeros(1)
+            cov.addCovariance(limix.CFixedCF(sp.ones((self.P,self.P))))
+            diag = sp.zeros(1)
         elif trait_covar_type=='block_id':
-            cov.addCovariance(limix.CFixedCF(SP.ones((self.P,self.P))))
-            cov.addCovariance(limix.CFixedCF(SP.eye(self.P)))
-            diag = SP.concatenate([SP.zeros(1),SP.ones(1)])
+            cov.addCovariance(limix.CFixedCF(sp.ones((self.P,self.P))))
+            cov.addCovariance(limix.CFixedCF(sp.eye(self.P)))
+            diag = sp.concatenate([sp.zeros(1),sp.ones(1)])
         elif trait_covar_type=='block_diag':
-            cov.addCovariance(limix.CFixedCF(SP.ones((self.P,self.P))))
+            cov.addCovariance(limix.CFixedCF(sp.ones((self.P,self.P))))
             cov.addCovariance(limix.CDiagonalCF(self.P))
-            diag = SP.concatenate([SP.zeros(1),SP.ones(self.P)])
+            diag = sp.concatenate([sp.zeros(1),sp.ones(self.P)])
         else:
             assert True==False, 'VarianceDecomposition:: trait_covar_type not valid'
         if jitter>0:
-            _cov = limix.CFixedCF(SP.eye(self.P))
-            _cov.setParams(SP.array([SP.sqrt(jitter)]))
-            _cov.setParamMask(SP.zeros(1))
+            _cov = limix.CFixedCF(sp.eye(self.P))
+            _cov.setParams(sp.array([sp.sqrt(jitter)]))
+            _cov.setParamMask(sp.zeros(1))
             cov.addCovariance(_cov)
         return cov,diag
         
@@ -768,14 +780,14 @@ class VarianceDecomposition:
         """
         verbose = limix.getVerbose(verbose)
         # Fit single trait model
-        varg  = SP.zeros(self.P)
-        varn  = SP.zeros(self.P)
-        fixed = SP.zeros((1,self.P))
+        varg  = sp.zeros(self.P)
+        varn  = sp.zeros(self.P)
+        fixed = sp.zeros((1,self.P))
         
         for p in range(self.P):
             y = self.Y[:,p:p+1]
             # check if some sull value
-            I  = SP.isnan(y[:,0])
+            I  = sp.isnan(y[:,0])
             if I.sum()>0:
                 y  = y[~I,:]
                 _K = K[~I,:][:,~I]
@@ -783,13 +795,13 @@ class VarianceDecomposition:
                 _K  = copy.copy(K)
             lmm = limix.CLMM()
             lmm.setK(_K)
-            lmm.setSNPs(SP.ones((y.shape[0],1)))
+            lmm.setSNPs(sp.ones((y.shape[0],1)))
             lmm.setPheno(y)
-            lmm.setCovs(SP.zeros((y.shape[0],1)))
+            lmm.setCovs(sp.zeros((y.shape[0],1)))
             lmm.setVarcompApprox0(-20, 20, 1000)
             lmm.process()
-            delta = SP.exp(lmm.getLdelta0()[0,0])
-            Vtot  = SP.exp(lmm.getLSigma()[0,0])
+            delta = sp.exp(lmm.getLdelta0()[0,0])
+            Vtot  = sp.exp(lmm.getLSigma()[0,0])
     
             varg[p] = Vtot
             varn[p] = delta*Vtot
@@ -819,8 +831,8 @@ class VarianceDecomposition:
         assert self.trait_covar_type[termx] not in ['lowrank','block','fixed'], 'VarianceDecimposition:: diagonal initializaiton not posible for such a parametrization'
         scales = []
         res = self._getH2singleTrait(self.vd.getTerm(termx).getK())
-        scaleg = SP.sqrt(res['varg'].mean())
-        scalen = SP.sqrt(res['varn'].mean())
+        scaleg = sp.sqrt(res['varg'].mean())
+        scalen = sp.sqrt(res['varn'].mean())
         for term_i in range(self.n_randEffs):
             if term_i==termx:
                 _scales = scaleg*self.diag[term_i]
@@ -829,9 +841,9 @@ class VarianceDecomposition:
             else:
                 _scales = 0.*self.diag[term_i]
             if self.jitter[term_i]>0:
-                _scales = SP.concatenate((_scales,SP.array([SP.sqrt(self.jitter[term_i])])))
+                _scales = sp.concatenate((_scales,sp.array([sp.sqrt(self.jitter[term_i])])))
             scales.append(_scales)
-        return SP.concatenate(scales)
+        return sp.concatenate(scales)
 
     def _getScalesRand(self):
         """
@@ -841,13 +853,13 @@ class VarianceDecomposition:
         if self.P>1:
             scales = []
             for term_i in range(self.n_randEffs):
-                _scales = SP.randn(self.diag[term_i].shape[0])
+                _scales = sp.randn(self.diag[term_i].shape[0])
                 if self.jitter[term_i]>0:
-                    _scales = SP.concatenate((_scales,SP.array([SP.sqrt(self.jitter[term_i])])))
+                    _scales = sp.concatenate((_scales,sp.array([sp.sqrt(self.jitter[term_i])])))
                 scales.append(_scales)
-            scales = SP.concatenate(scales)
+            scales = sp.concatenate(scales)
         else:
-            scales=SP.randn(self.vd.getNumberScales())
+            scales=sp.randn(self.vd.getNumberScales())
         return scales
 
     def _perturbation(self):
@@ -858,13 +870,13 @@ class VarianceDecomposition:
         if self.P>1:
             scales = []
             for term_i in range(self.n_randEffs):
-                _scales = SP.randn(self.diag[term_i].shape[0])
+                _scales = sp.randn(self.diag[term_i].shape[0])
                 if self.jitter[term_i]>0:
-                    _scales  = SP.concatenate((_scales,SP.zeros(1)))
+                    _scales  = sp.concatenate((_scales,sp.zeros(1)))
                 scales.append(_scales)
-            scales = SP.concatenate(scales)
+            scales = sp.concatenate(scales)
         else:
-            scales = SP.randn(self.vd.getNumberScales())
+            scales = sp.randn(self.vd.getNumberScales())
         return scales
 
     """ INTERNAL FUNCIONS FOR ESTIMATING PARAMETERS UNCERTAINTY """
@@ -879,7 +891,7 @@ class VarianceDecomposition:
         
         if self.cache['Hessian']==None:
             ParamMask=self.gp.getParamMask()['covar']
-            std=SP.zeros(ParamMask.sum())
+            std=sp.zeros(ParamMask.sum())
             H=self.gp.LMLhess_covar()
             It= (ParamMask[:,0]==1)
             self.cache['Hessian']=H[It,:][:,It]
@@ -897,7 +909,7 @@ class VarianceDecomposition:
         assert self.fast==False, 'Not supported for fast implementation'
         
         if self.cache['Sigma']==None:
-            self.cache['Sigma'] = SP.linalg.inv(self.getHessian())
+            self.cache['Sigma'] = sp.linalg.inv(self._getHessian())
         return self.cache['Sigma']
 
     def _getFisher(self):
@@ -906,13 +918,13 @@ class VarianceDecomposition:
         TO CHECK: IT DOES NOT WORK PROPERLY
         """
         Ctot = self.vd.getGP().getCovar()
-        Ki = SP.linalg.inv(Ctot.K())
+        Ki = sp.linalg.inv(Ctot.K())
         n_scales = self.vd.getNumberScales()
-        out = SP.zeros((n_scales,n_scales))
+        out = sp.zeros((n_scales,n_scales))
         for m in range(n_scales):
-            out[m,m] = 0.5 * SP.trace(SP.dot(Ki,SP.dot(Ctot.Kgrad_param(m),SP.dot(Ki,Ctot.Kgrad_param(m)))))
+            out[m,m] = 0.5 * sp.trace(sp.dot(Ki,sp.dot(Ctot.Kgrad_param(m),sp.dot(Ki,Ctot.Kgrad_param(m)))))
             for n in range(m):
-                out[m,n]=0.5 * SP.trace(SP.dot(Ki,SP.dot(Ctot.Kgrad_param(m),SP.dot(Ki,Ctot.Kgrad_param(n)))))
+                out[m,n]=0.5 * sp.trace(sp.dot(Ki,sp.dot(Ctot.Kgrad_param(m),sp.dot(Ki,Ctot.Kgrad_param(n)))))
                 out[n,m]=out[m,n]
         return out
 
@@ -922,9 +934,9 @@ class VarianceDecomposition:
         """
         USES LAPLACE APPROXIMATION TO CALCULATE THE BAYESIAN MODEL POSTERIOR
         """
-        Sigma = self.getLaplaceCovar(min)
+        Sigma = self._getLaplaceCovar(min)
         n_params = self.vd.getNumberScales()
-        ModCompl = 0.5*n_params*SP.log(2*SP.pi)+0.5*SP.log(SP.linalg.det(Sigma))
+        ModCompl = 0.5*n_params*sp.log(2*sp.pi)+0.5*sp.log(sp.linalg.det(Sigma))
         RV = min['LML']+ModCompl
         return RV
 
